import os
import pandas as pd
from flask import Flask, request, render_template, redirect, url_for, flash, send_from_directory
from werkzeug.utils import secure_filename
import io # Required for capturing pandas info output

# Configuration
UPLOAD_FOLDER = 'uploads'
ALLOWED_EXTENSIONS = {'csv'}
MAX_CONTENT_LENGTH = 16 * 1024 * 1024  # 16 MB limit for uploads

app = Flask(__name__)
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
app.config['MAX_CONTENT_LENGTH'] = MAX_CONTENT_LENGTH
app.secret_key = 'super secret key' # Change this in production!

# --- Helper Function ---
def allowed_file(filename):
    """Checks if the filename has an allowed extension."""
    return '.' in filename and \
           filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

# --- Ensure Upload Folder Exists ---
if not os.path.exists(UPLOAD_FOLDER):
    os.makedirs(UPLOAD_FOLDER)
    print(f"Created upload directory: {UPLOAD_FOLDER}")

# --- Routes ---
@app.route('/', methods=['GET'])
def index():
    """Renders the upload form page."""
    # Calculate the max size in MB from the app's config
    # Use .get() with a default to avoid errors if the key is missing
    max_size_bytes = app.config.get('MAX_CONTENT_LENGTH', 0)
    max_size_mb = int(max_size_bytes / 1024 / 1024) if max_size_bytes else 0

    # Pass the calculated value to the template
    return render_template('index.html', max_upload_size_mb=max_size_mb) # Changed here

@app.route('/upload', methods=['POST'])
def upload_file():
    """Handles file upload and processing."""
    if 'file' not in request.files:
        flash('No file part in the request.', 'error')
        return redirect(request.url) # Redirect back to the same URL (which is '/')

    file = request.files['file']

    if file.filename == '':
        flash('No selected file.', 'error')
        return redirect(url_for('index')) # Redirect to the index page

    if file and allowed_file(file.filename):
        filename = secure_filename(file.filename)
        filepath = os.path.join(app.config['UPLOAD_FOLDER'], filename)

        try:
            file.save(filepath)
            flash(f'File "{filename}" uploaded successfully!', 'success')

            # --- Process the CSV and Generate Analytics ---
            try:
                df = pd.read_csv(filepath)

                # Basic Info
                num_rows, num_cols = df.shape
                column_names = df.columns.tolist()
                data_types = df.dtypes.to_dict() # Get data types as a dictionary

                # Capture df.info() output
                buffer = io.StringIO()
                df.info(buf=buffer)
                info_summary = buffer.getvalue()

                # Descriptive Statistics (convert to HTML)
                # Only include numeric columns for describe()
                numeric_cols = df.select_dtypes(include=['number'])
                if not numeric_cols.empty:
                    description_html = numeric_cols.describe().to_html(classes='table table-striped table-bordered', border=0)
                else:
                    description_html = "<p>No numeric columns found for descriptive statistics.</p>"


                # First 5 rows (convert to HTML)
                head_html = df.head().to_html(classes='table table-striped table-bordered', border=0, index=False)

                # --- Prepare data for template ---
                analysis_results = {
                    'filename': filename,
                    'num_rows': num_rows,
                    'num_cols': num_cols,
                    'column_names': column_names,
                    'data_types': data_types, # Pass the dict
                    'info_summary': info_summary,
                    'description_html': description_html,
                    'head_html': head_html
                }

                return render_template('results.html', results=analysis_results)

            except pd.errors.EmptyDataError:
                flash(f'Error: The uploaded CSV file "{filename}" is empty.', 'error')
                # Optionally remove the empty file
                # os.remove(filepath)
                return redirect(url_for('index'))
            except pd.errors.ParserError:
                 flash(f'Error: Could not parse the CSV file "{filename}". Check its format.', 'error')
                 # Optionally remove the problematic file
                 # os.remove(filepath)
                 return redirect(url_for('index'))
            except Exception as e:
                flash(f'An error occurred during analysis: {str(e)}', 'error')
                # Optionally remove the file if analysis failed
                # os.remove(filepath)
                return redirect(url_for('index'))

        except Exception as e:
            # This catches errors during file saving (e.g., permissions)
            flash(f'An error occurred while saving the file: {str(e)}', 'error')
            return redirect(url_for('index'))

    else:
        flash('Invalid file type. Only .csv files are allowed.', 'error')
        return redirect(url_for('index'))

# Optional: Add route to view/download uploaded files if needed (use with caution)
# @app.route('/uploads/<filename>')
# def uploaded_file(filename):
#    try:
#        return send_from_directory(app.config['UPLOAD_FOLDER'], filename)
#    except FileNotFoundError:
#        flash(f"File '{filename}' not found.", 'error')
#        return redirect(url_for('index'))


# --- Run the Application ---
if __name__ == '__main__':
    # Set debug=True for development (shows detailed errors)
    # Set debug=False for production
    app.run(debug=True, host='0.0.0.0', port=5050)